# Generated by roxygen2: do not edit by hand

S3method(print,life_table)
S3method(print,select_life_table)
export(A2barx)
export(A2barxn)
export(A2barxn1)
export(A2nAbarx)
export(A2nAx)
export(A2nAx_m)
export(A2nEx)
export(A2x)
export(A2x_m)
export(A2xn)
export(A2xn1)
export(A2xn1_m)
export(A2xn_m)
export(AAL_PUC_db)
export(AAL_TUC_db)
export(AB_cae)
export(AB_fas)
export(APV_NR_db)
export(APV_gross_premiums)
export(AS_path)
export(AS_path_md)
export(AV_path_ul_typeA)
export(AV_path_ul_typeB)
export(AVz_dc)
export(Abarx)
export(Abarx_udd)
export(Abarxj_md)
export(Abarxn)
export(Abarxn1)
export(Abarxn1_udd)
export(Abarxn_udd)
export(Abarxy)
export(Abarxy1)
export(Abarxy2)
export(Abarxybar)
export(Abaryx1)
export(Abaryx2)
export(Ax)
export(Ax_m)
export(Ax_m_udd)
export(Axj_md)
export(Axn)
export(Axn1)
export(Axn1_m)
export(Axn1_m_udd)
export(Axn1_spot)
export(Axn1_var)
export(Axn_m)
export(Axn_m_udd)
export(Axn_spot)
export(Axn_var)
export(Axy)
export(Axybar)
export(Axybarn)
export(Axybarn1)
export(Axyn)
export(Axyn1)
export(DAbarxn1)
export(DAxn1)
export(Dabarxn)
export(Dadotxn)
export(Daxn)
export(DbarAbarxn1)
export(EL0barAbarx)
export(EL0x)
export(EL0xn)
export(EL0xn1)
export(ELtx)
export(F0)
export(GI_cont)
export(GI_disc)
export(GMF_rollforward_ul)
export(GM_cont)
export(GM_disc)
export(GT_cont)
export(GT_disc)
export(GTg_disc)
export(Gx)
export(IAbarx)
export(IAx)
export(IAxn1)
export(IRR_profit)
export(Iabarx)
export(Iabarxn)
export(Iadotx)
export(Iadotxn)
export(Iax)
export(Iaxn)
export(IbarAbarx)
export(IbarAbarxn1)
export(Income_dc)
export(NC_EAN_db)
export(NC_PUC_db)
export(NC_TUC_db)
export(NPV_partial)
export(NPV_profit)
export(PAB_cae)
export(PAB_fas)
export(PbarAbarx)
export(PbarAbarxn)
export(PbarAbarxn1)
export(Pbar_trapz_ms)
export(Pbarx)
export(Pbarxn)
export(Pbarxn1)
export(Pi_signature)
export(PnAdotx)
export(PnAx)
export(PnAx_m)
export(PnEx)
export(Pnax)
export(Pr_vector_disc)
export(Px)
export(Px_m)
export(Pxn)
export(Pxn1)
export(Pxn1_m)
export(Pxn_m)
export(S0)
export(S0_to_lx)
export(V_zeroized)
export(Vprefloor_crvm_ul)
export(abarx)
export(abarx_udd)
export(abarx_woolhouse2)
export(abarx_woolhouse3)
export(abarx_y)
export(abarxn)
export(abarxn_udd)
export(abarxy)
export(abarxybar)
export(abary_x)
export(adotx)
export(adotx_m)
export(adotx_m_udd)
export(adotx_m_woolhouse2)
export(adotx_m_woolhouse3)
export(adotxn)
export(adotxn_m)
export(adotxn_m_udd)
export(adotxn_m_woolhouse2)
export(adotxn_m_woolhouse3)
export(adotxy)
export(adotxybar)
export(adotxybarn)
export(adotxyn)
export(ag38_prefunding_ratio)
export(ag38_reserve_ul)
export(alphaF)
export(annuity_certain)
export(annuity_identity_abarx)
export(annuity_identity_abarxn)
export(annuity_identity_adotx)
export(annuity_identity_adotxn)
export(annuity_identity_ax)
export(annuity_identity_axn)
export(annuity_identity_nabarx)
export(annuity_identity_nadotx)
export(annuity_identity_nax)
export(ax)
export(ax_improved)
export(ax_m)
export(ax_m_udd)
export(ax_m_woolhouse2)
export(ax_m_woolhouse3)
export(ax_y)
export(axn)
export(axn_improved)
export(axn_m)
export(axn_m_udd)
export(axn_m_woolhouse2)
export(axn_m_woolhouse3)
export(axn_spot)
export(axn_var)
export(axy)
export(axybar)
export(axybarn)
export(axyn)
export(ay_x)
export(betaF)
export(coi_ul_typeB)
export(contribution_rate_target)
export(cov_term_deferred)
export(cov_term_endow)
export(cumhaz0)
export(decompGg_disc)
export(discount)
export(discounted_payback_period)
export(double_force_delta)
export(double_force_i)
export(dx)
export(dxj)
export(dxtau)
export(ex_complete)
export(ex_complete_tab)
export(ex_curtate)
export(ex_curtate_tab)
export(ex_temp_complete_tab)
export(ex_temp_curtate_tab)
export(f0)
export(fnk_from_z)
export(forward_matrix_from_z)
export(fx)
export(fx_tab)
export(gain_loss_md)
export(hazard0)
export(htVnAx)
export(htVx)
export(iMA_eiul)
export(iP_eiul)
export(i_credit_eiul)
export(interest_convert)
export(life_table)
export(lx)
export(lx_select)
export(lx_to_S0)
export(markov_nstep_prob)
export(md_table)
export(meanVx)
export(mux_tab)
export(nAbarx)
export(nAbarx_udd)
export(nAx)
export(nAx_m)
export(nAx_m_udd)
export(nEx)
export(nEx_spot)
export(nEx_var)
export(nExy)
export(nExybar)
export(nabarx)
export(nabarx_udd)
export(nadotx)
export(nadotx_m)
export(nadotx_m_udd)
export(nadotx_m_woolhouse2)
export(nadotx_m_woolhouse3)
export(nax)
export(nax_m)
export(nax_m_udd)
export(nax_m_woolhouse2)
export(nax_m_woolhouse3)
export(naxn_improved)
export(ndx)
export(nkqx)
export(nmxq)
export(nmxq_select)
export(npx)
export(npx_select)
export(npxtau_md)
export(nqx)
export(nqx_select)
export(nqxj_md)
export(nqxtau_md)
export(profit_margin)
export(pv_cashflows)
export(pv_spot_cashflows)
export(px_proj)
export(px_to_lx)
export(pxtau)
export(pxtau_ul)
export(qx_dep_cf)
export(qx_dep_sudd)
export(qx_proj)
export(qx_tab)
export(qx_to_lx)
export(qxprime_mudd)
export(qxprime_sudd)
export(qxtau)
export(replacement_ratio_db)
export(replacement_ratio_dc)
export(rt_ul)
export(salary_scale)
export(sbarxn)
export(sdotxn)
export(sdotxn_m)
export(sdotxn_m_udd)
export(sdotxn_m_woolhouse2)
export(select_life_table)
export(solve_yield)
export(sxn)
export(sxn_m)
export(sxn_m_udd)
export(sxn_m_woolhouse2)
export(tPnAx)
export(tPnEx)
export(tPx)
export(tPxn)
export(tPxn1)
export(tVEx)
export(tVFx)
export(tVGx)
export(tVbarAbarx)
export(tVbarx)
export(tVnAdotx)
export(tVnAx)
export(tVnEx)
export(tVnax)
export(tVx)
export(tVx_m)
export(tVx_ret)
export(tVxn)
export(tVxn1)
export(tVxn1_ret)
export(tVxn_ret)
export(thiele_backward_path)
export(thiele_backward_step)
export(thiele_dVdt)
export(thiele_dVdt_01)
export(thiele_path_01)
export(tp00_tp01_euler)
export(tpx)
export(tpx_improved)
export(tpx_tab)
export(tpx_tau_cf)
export(tpxprimej_cf)
export(tpxtau_ul)
export(tpxy)
export(tpxybar)
export(tqx)
export(tqx_tab)
export(tqxj_cf)
export(tqxprime_mudd)
export(tqxprimej_cf)
export(tqxy)
export(tqxy1)
export(tqxy2)
export(tqxybar)
export(tqyx1)
export(tqyx2)
export(tsVx)
export(tsVxn)
export(tsVxn1)
export(udd_continuous_multiplier)
export(udd_mthly_multiplier)
export(varL0barAbarx)
export(varL0x)
export(varL0xn)
export(varL0xn1)
export(varLtx)
export(var_Abarx)
export(var_Abarxn)
export(var_Abarxn1)
export(var_Ax)
export(var_Ax_m)
export(var_Axn)
export(var_Axn1)
export(var_Axn1_m)
export(var_Axn_m)
export(var_nAbarx)
export(var_nAx)
export(var_nAx_m)
export(var_nEx)
export(vt_var)
export(z_from_coupon_annual)
export(z_from_coupon_semi)
export(z_from_fn1)
importFrom(stats,integrate)
importFrom(stats,uniroot)
importFrom(utils,head)
importFrom(utils,tail)
