#' mqriskR: Models for Quantifying Risk
#'
#' Actuarial functions for survival models, life contingencies, reserves,
#' multiple-decrement models, variable interest, universal life, profit
#' analysis, and pension mathematics.
#'
#' @importFrom stats integrate uniroot
#' @importFrom utils head tail
#' @keywords internal
"_PACKAGE"
