% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multilife_ch12.R
\name{reversionary_annuities}
\alias{reversionary_annuities}
\alias{ax_y}
\alias{ay_x}
\title{Reversionary annuity functions}
\usage{
ax_y(x, y, i, tbl = NULL, model = NULL, ..., k_max = 5000L, tol = 1e-12)

ay_x(x, y, i, tbl = NULL, model = NULL, ..., k_max = 5000L, tol = 1e-12)
}
\arguments{
\item{x}{Age of the first life. May be scalar or vector.}

\item{y}{Age of the second life. May be scalar or vector.}

\item{i}{Effective annual interest rate. May be scalar or vector.}

\item{tbl}{Optional life table object.}

\item{model}{Optional parametric survival model.}

\item{...}{Additional parameters passed to the survival model or
life-table functions.}

\item{k_max}{Maximum number of terms used for an infinite series.}

\item{tol}{Numerical tolerance used to assess convergence.}
}
\value{
A numeric vector of actuarial present values.
}
\description{
Computes reversionary whole-life annuities payable to one life after the
death of the other life.
}
\details{
\code{ax_y()} computes an annuity payable to the second life after the
death of the first life.

\code{ay_x()} computes an annuity payable to the first life after the
death of the second life.
}
