% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reserve_ch11.R, R/reserve_ch11_ext.R
\name{tVFx}
\alias{tVFx}
\title{Full preliminary term reserve for whole life insurance}
\usage{
tVFx(x, t, i, tbl = NULL, model = NULL, ...)

tVFx(x, t, i, tbl = NULL, model = NULL, ...)
}
\arguments{
\item{x}{Issue age.}

\item{t}{Duration.}

\item{i}{Effective annual interest rate.}

\item{tbl}{Optional life table object.}

\item{model}{Optional parametric survival model.}

\item{...}{Additional model parameters.}
}
\value{
Numeric vector.

Numeric vector.
}
\description{
Computes the FPT reserve for a whole life insurance.
For whole life insurance,
\eqn{{}_1V^F = 0}
and for \eqn{t \ge 1},
\eqn{{}_tV^F = {}_{t-1}V_{x+1}^{NLP}}.

Computes the FPT reserve for a whole life insurance.
For whole life insurance,
\eqn{{}_1V^F = 0}
and for \eqn{t \ge 1},
\eqn{{}_tV^F = {}_{t-1}V_{x+1}^{NLP}}.
}
\examples{
tVFx(40, t = 5, i = 0.05, model = "uniform", omega = 100)
tVFx(40, t = 5, i = 0.05, model = "uniform", omega = 100)
}
