% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annuity_mthly.R
\name{annuity_mthly_deferred_immediate}
\alias{annuity_mthly_deferred_immediate}
\alias{nax_m}
\title{Deferred whole life m-thly annuity-immediate}
\usage{
nax_m(x, n, m, i, model, ..., k_max = 2e+05, tol = 1e-12)
}
\arguments{
\item{x}{Age.}

\item{n}{Deferral period in years.}

\item{m}{Number of payments per year.}

\item{i}{Effective annual interest rate.}

\item{model}{Survival model name.}

\item{...}{Additional parameters passed to the survival model.}

\item{k_max}{Maximum summation horizon for non-terminating models.}

\item{tol}{Truncation tolerance for non-terminating models.}
}
\value{
Numeric vector of annuity values.
}
\description{
Computes the deferred whole life m-thly annuity-immediate using
\deqn{{}_nE_x \, a_{x+n}^{(m)}}
}
\examples{
nax_m(40, n = 10, m = 12, i = 0.05, model = "uniform", omega = 100)
}
