% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mortality_improvement.R
\name{px_proj}
\alias{px_proj}
\title{Project one-year survival probability under mortality improvement}
\usage{
px_proj(qx_base, AAx, base_year, proj_year)
}
\arguments{
\item{qx_base}{Base-year one-year death probability \eqn{q_x^{[B]}}.}

\item{AAx}{Mortality improvement factor \eqn{AA_x}.}

\item{base_year}{Base year \eqn{B}.}

\item{proj_year}{Projection year \eqn{Y}. May be scalar or vector.}
}
\value{
Numeric vector of projected one-year survival probabilities.
}
\description{
Computes projected one-year survival probability
\deqn{p_x^{[Y]} = 1 - q_x^{[Y]}}.
}
