% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annuity_relationships.R
\name{annuity_relationships}
\alias{annuity_relationships}
\alias{annuity_identity_ax}
\alias{annuity_identity_adotx}
\alias{annuity_identity_abarx}
\alias{annuity_identity_axn}
\alias{annuity_identity_adotxn}
\alias{annuity_identity_abarxn}
\alias{annuity_identity_nax}
\alias{annuity_identity_nadotx}
\alias{annuity_identity_nabarx}
\title{Annuity-insurance relationships}
\usage{
annuity_identity_ax(x, i, model = NULL, ..., tbl = NULL)

annuity_identity_adotx(x, i, model = NULL, ..., tbl = NULL)

annuity_identity_abarx(x, i, model, ...)

annuity_identity_axn(x, n, i, model = NULL, ..., tbl = NULL)

annuity_identity_adotxn(x, n, i, model = NULL, ..., tbl = NULL)

annuity_identity_abarxn(x, n, i, model, ...)

annuity_identity_nax(
  x,
  n,
  i,
  model = NULL,
  ...,
  tbl = NULL,
  k_max = 5000,
  tol = 1e-12
)

annuity_identity_nadotx(
  x,
  n,
  i,
  model = NULL,
  ...,
  tbl = NULL,
  k_max = 5000,
  tol = 1e-12
)

annuity_identity_nabarx(x, n, i, model, ..., tol = 1e-10)
}
\arguments{
\item{x}{Age.}

\item{i}{Effective annual interest rate.}

\item{model}{Optional survival model name.}

\item{...}{Additional model parameters.}

\item{tbl}{Optional life table object for discrete identities.}

\item{n}{Term or deferral period in years.}

\item{k_max}{Maximum summation horizon for non-terminating models.}

\item{tol}{Truncation tolerance for non-terminating models.}
}
\value{
Numeric vector containing the annuity value computed from the corresponding
annuity-insurance identity.
}
\description{
Identities linking annual and continuous annuity functions to the
corresponding insurance functions.

Computes \eqn{a_x = (v - A_x)/d}.

Computes \eqn{\ddot{a}_x = (1 - A_x)/d}.

Computes \eqn{\bar{a}_x = (1 - \bar{A}_x)/\delta}.

Computes
\eqn{a_{x:\overline{n}|} = (1 - A_{x:\overline{n}|})/d - 1 + {}_nE_x}.

Computes \eqn{\ddot{a}_{x:\overline{n}|} =
(1 - A_{x:\overline{n}|})/d}.

Computes \eqn{\bar{a}_{x:\overline{n}|} =
(1 - \bar{A}_{x:\overline{n}|})/\delta}.

Computes \eqn{{}_{n|}a_x = {}_nE_x a_{x+n}}.

Computes \eqn{{}_{n|}\ddot{a}_x = {}_nE_x \ddot{a}_{x+n}}.

Computes \eqn{{}_{n|}\bar{a}_x = {}_nE_x \bar{a}_{x+n}}.
}
