% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reserve_ch10.R
\name{tVbarx}
\alias{tVbarx}
\title{Whole life reserve with continuous premiums}
\usage{
tVbarx(x, t, i, model = NULL, ..., tbl = NULL)
}
\arguments{
\item{x}{Issue age.}

\item{t}{Duration.}

\item{i}{Effective annual interest rate.}

\item{model}{Optional parametric survival model name.}

\item{...}{Additional model parameters.}

\item{tbl}{Optional life table object.}
}
\value{
A numeric vector of values.
}
\description{
Computes the reserve for a discrete whole life insurance
funded by continuous premiums.
}
\examples{
tVbarx(40, t = 10, i = 0.05, model = "uniform", omega = 100)
}
