% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survival_models.R
\name{fx}
\alias{fx}
\title{Conditional density}
\usage{
fx(t, x, model, ...)
}
\arguments{
\item{t}{Numeric vector of durations.}

\item{x}{Numeric vector of ages.}

\item{model}{One of \code{"uniform"}, \code{"exponential"},
\code{"gompertz"}, \code{"makeham"}, or \code{"weibull"}.}

\item{...}{Model parameters.}
}
\value{
Numeric vector of density values.
}
\description{
Computes \eqn{f_x(t)=f_0(x+t)/S_0(x)}.
}
