% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chapter15_variable_interest.R
\name{z_from_coupon_annual}
\alias{z_from_coupon_annual}
\title{Bootstrap annual effective spot rates}
\usage{
z_from_coupon_annual(maturity, coupon_yield, par = 1000)
}
\arguments{
\item{maturity}{Numeric vector of positive integer maturities in strictly
increasing order. Maturities must be consecutive and begin at 1.}

\item{coupon_yield}{Numeric vector of annual effective par coupon yields.
Values must be greater than \code{-1}.}

\item{par}{Positive scalar par value.}
}
\value{
A numeric vector of annual effective spot rates.
}
\description{
Bootstraps annual effective spot rates from par coupon yields at consecutive
integer maturities.
}
\examples{
maturity <- 1:4
coupon_yield <- c(0.02, 0.04, 0.06, 0.08)
z_from_coupon_annual(maturity, coupon_yield)

}
