% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chapter14_multiple_decrement.R
\name{Axj_md}
\alias{Axj_md}
\title{Discrete multiple-decrement insurance present value}
\usage{
Axj_md(qj, ptau, i, benefit = 1)
}
\arguments{
\item{qj}{Cause-specific decrement probabilities by policy year.}

\item{ptau}{In-force probabilities at the beginning of each policy year.}

\item{i}{Effective annual interest rate. May be scalar or vector.}

\item{benefit}{Benefit payable on decrement. May be scalar or vector.}
}
\value{
A numeric vector of actuarial present values.
}
\description{
Computes the actuarial present value of a benefit payable at the end of
the year of decrement from a specified cause.
}
\examples{
qj <- c(0.02, 0.02, 0.02, 0.02, 0.02)
ptau <- c(1, 0.95, 0.89, 0.82, 0.74)
Axj_md(qj, ptau, i = 0.06, benefit = 1000)

}
