% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decrement_ch13_ext.R
\name{qxprime_mudd}
\alias{qxprime_mudd}
\title{Associated single-decrement probabilities under MUDD}
\usage{
qxprime_mudd(qxj)
}
\arguments{
\item{qxj}{Numeric vector of dependent cause-specific decrement
probabilities.}
}
\value{
A numeric vector of associated single-decrement probabilities.
}
\description{
Converts dependent multiple-decrement probabilities to associated
single-decrement probabilities under the multiple-decrement uniform
distribution assumption.
}
\examples{
qxprime_mudd(c(0.20, 0.10))

}
