% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/insurance_continuous.R
\name{Abarx}
\alias{Abarx}
\title{Continuous whole life insurance APV}
\usage{
Abarx(x, i, model, ...)
}
\arguments{
\item{x}{Age.}

\item{i}{Effective annual interest rate.}

\item{model}{Parametric survival model name.}

\item{...}{Additional model parameters passed to survival-model functions.}
}
\value{
Numeric vector of APVs.
}
\description{
Computes
\eqn{\bar{A}_x = \int_0^\infty v^t {}_t p_x \mu_{x+t}\,dt}.
}
