% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chapter15_variable_interest.R
\name{z_from_coupon_semi}
\alias{z_from_coupon_semi}
\title{Bootstrap semiannual nominal spot rates}
\usage{
z_from_coupon_semi(maturity, coupon_yield, par = 1000)
}
\arguments{
\item{maturity}{Numeric vector of positive maturities in years, in strictly
increasing order. Maturities must be consecutive multiples of
\code{0.5}.}

\item{coupon_yield}{Numeric vector of nominal annual par coupon yields
convertible semiannually. Values must be greater than \code{-2}.}

\item{par}{Positive scalar par value.}
}
\value{
A numeric vector of nominal annual spot rates convertible
  semiannually.
}
\description{
Bootstraps nominal annual spot rates convertible semiannually from par
coupon yields at consecutive half-year maturities.
}
\examples{
maturity <- c(0.5, 1.0, 1.5, 2.0)
coupon_yield <- c(0.0244, 0.0260, 0.0276, 0.0293)
z_from_coupon_semi(maturity, coupon_yield)

}
