% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/life_table_core.R
\name{nqx}
\alias{nqx}
\title{Compute n-year death probability from a life table}
\usage{
nqx(tbl, x, n)
}
\arguments{
\item{tbl}{A \code{life_table} object.}

\item{x}{Ages.}

\item{n}{Nonnegative integer durations.}
}
\value{
Numeric vector of \eqn{{}_n q_x} values.
}
\description{
Compute n-year death probability from a life table
}
