% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/insurance_continuous.R
\name{A2barxn1}
\alias{A2barxn1}
\title{Second moment of continuous term insurance PV}
\usage{
A2barxn1(x, n, i, model, ...)
}
\arguments{
\item{x}{Age.}

\item{n}{Term.}

\item{i}{Effective annual interest rate.}

\item{model}{Parametric survival model name.}

\item{...}{Additional model parameters passed to survival-model functions.}
}
\value{
Numeric vector of second moments.
}
\description{
Computes \eqn{{}^{2}\bar{A}_{x:\overline{n}|}^{1}} by evaluating
\eqn{\bar{A}_{x:\overline{n}|}^{1}} at doubled force.
}
