% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chapter16_universal_life.R
\name{iMA_eiul}
\alias{iMA_eiul}
\title{Monthly-average index growth rate}
\usage{
iMA_eiul(index)
}
\arguments{
\item{index}{Numeric vector of length 13 containing a strictly positive
initial index value followed by twelve nonnegative monthly closing values.}
}
\value{
A numeric scalar.
}
\description{
Computes the monthly-average growth rate from an initial index value and
twelve monthly closing values.
}
\examples{
index <- c(
  1000, 1020, 1100, 1150, 1080, 1040, 960,
  1030, 1000, 1070, 1150, 1200, 1150
)
iMA_eiul(index)

}
