% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chapter18_pensions.R
\name{contribution_rate_target}
\alias{contribution_rate_target}
\title{Target contribution rate for a defined contribution plan}
\usage{
contribution_rate_target(x, z, Sx, RR_target, i, adue_z, g = NULL, s = NULL)
}
\arguments{
\item{x}{Scalar entry age.}

\item{z}{Scalar retirement age.}

\item{Sx}{Positive scalar salary at age \code{x}.}

\item{RR_target}{Scalar target replacement ratio in \code{[0, 1]}.}

\item{i}{Scalar annual effective investment return greater than \code{-1}.}

\item{adue_z}{Positive scalar whole-life annuity-due factor at retirement.}

\item{g}{Optional scalar annual salary growth rate greater than \code{-1}.}

\item{s}{Optional positive salary-scale vector of length \code{z - x}.}
}
\value{
A numeric scalar. The result may exceed one when the target cannot
  be achieved with a contribution rate no greater than 100 percent.
}
\description{
Calculates the contribution rate required to achieve a target replacement
ratio.
}
\examples{
contribution_rate_target(
  x = 30,
  z = 65,
  Sx = 60000,
  RR_target = 0.50,
  i = 0.06,
  adue_z = 11,
  g = 0.04
)

}
