% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chapter18_pensions.R
\name{AB_fas}
\alias{AB_fas}
\title{Accrued benefit under a final-average-salary plan}
\usage{
AB_fas(salary_history, p, fas_years = 3)
}
\arguments{
\item{salary_history}{Positive numeric vector of annual salaries.}

\item{p}{Nonnegative scalar accrual percentage.}

\item{fas_years}{Positive integer number of years in the salary average.}
}
\value{
A numeric scalar.
}
\description{
Computes the accrued benefit using salary and service history through the
valuation date.
}
\examples{
AB_fas(
  salary_history = c(150000, 156000),
  p = 1,
  fas_years = 2
)

}
