% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/insurance_varying_benefits.R
\name{DbarAbarxn1}
\alias{DbarAbarxn1}
\title{Fully continuous decreasing n-year term insurance}
\usage{
DbarAbarxn1(x, n, i, model, ...)
}
\arguments{
\item{x}{Age.}

\item{n}{Term.}

\item{i}{Effective annual interest rate.}

\item{model}{Parametric survival model.}

\item{...}{Additional model parameters.}
}
\value{
Numeric vector.
}
\description{
Computes
\deqn{(\bar{D}\bar{A})_{x:\overline{n}|}^{1}
= \int_0^n (n-t) v^t {}_tp_x \mu_{x+t}\,dt.}
}
