% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/insurance_mthly.R
\name{nAx_m}
\alias{nAx_m}
\title{m-thly deferred insurance APV}
\usage{
nAx_m(x, n, i, m, model, ..., tol = 1e-12, j_max = 100000L)
}
\arguments{
\item{x}{Age.}

\item{n}{Deferral period.}

\item{i}{Effective annual interest rate.}

\item{m}{Positive integer payment frequency.}

\item{model}{Parametric survival model name.}

\item{...}{Additional model parameters passed to survival-model functions.}

\item{tol}{Numerical tolerance for truncating the infinite sum.}

\item{j_max}{Maximum number of m-thly intervals in the sum.}
}
\value{
Numeric vector of APVs.
}
\description{
Computes
\eqn{{}_{n\mid}A_x^{(m)} = v^n {}_np_x A_{x+n}^{(m)}}.
}
