% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reserve_ch10.R
\name{htVx}
\alias{htVx}
\title{h-pay whole life net level premium reserve}
\usage{
htVx(x, h, t, i, model = NULL, ..., tbl = NULL)
}
\arguments{
\item{x}{Issue age.}

\item{h}{Premium-paying period in years.}

\item{t}{Duration.}

\item{i}{Effective annual interest rate.}

\item{model}{Optional parametric survival model name.}

\item{...}{Additional model parameters.}

\item{tbl}{Optional life table object.}
}
\value{
A numeric vector of values.
}
\description{
Computes the prospective reserve for an h-pay whole life policy.
}
\examples{
htVx(40, h = 10, t = 5, i = 0.05, model = "uniform", omega = 100)
}
