% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reserve_ch11.R
\name{fractional_duration_term_endowment_reserves}
\alias{fractional_duration_term_endowment_reserves}
\alias{tsVxn}
\alias{tsVxn1}
\title{Fractional-duration term and endowment reserves}
\usage{
tsVxn(x, n, t, s, i, tbl = NULL, model = NULL, ...)

tsVxn1(x, n, t, s, i, tbl = NULL, model = NULL, ...)
}
\arguments{
\item{x}{Issue age. May be scalar or vector.}

\item{n}{Positive integer term. May be scalar or vector.}

\item{t}{Nonnegative integer duration satisfying \eqn{t < n}. May be
scalar or vector.}

\item{s}{Fractional duration in \eqn{[0,1]}. May be scalar or vector.}

\item{i}{Effective annual interest rate. May be scalar or vector.}

\item{tbl}{Optional life table object.}

\item{model}{Optional parametric survival model.}

\item{...}{Additional parameters passed to the actuarial functions.}
}
\value{
A numeric vector of reserve values.
}
\description{
Computes fractional-duration reserves for term and endowment insurance
using linear interpolation between the reserve immediately after the
premium at duration \eqn{t} and the reserve at duration \eqn{t+1}.
}
\details{
\code{tsVxn()} computes the fractional-duration reserve for endowment
insurance.

\code{tsVxn1()} computes the fractional-duration reserve for term
insurance.


The fractional reserve is computed using

\deqn{
{}_{t+s}V
=
({}_tV + P)(1-s)
+
{}_{t+1}V s.
}

The premium \eqn{P} is the net annual premium for the corresponding
insurance contract (term or endowment).
}
\examples{
tsVxn(
  40,
  n = 20,
  t = 10,
  s = 0.5,
  i = 0.05,
  model = "uniform",
  omega = 100
)

tsVxn1(
  40,
  n = 20,
  t = 10,
  s = 0.5,
  i = 0.05,
  model = "uniform",
  omega = 100
)

}
