% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/insurance_continuous.R
\name{insurance_continuous}
\alias{insurance_continuous}
\title{Continuous insurance models}
\description{
Continuous contingent payment and insurance functions.
}
\keyword{internal}
