% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annuity_varying_payments.R
\name{annuity_varying_payments}
\alias{annuity_varying_payments}
\alias{Iax}
\alias{Iaxn}
\alias{Daxn}
\alias{Iadotx}
\alias{Iadotxn}
\alias{Dadotxn}
\alias{Iabarx}
\alias{Iabarxn}
\alias{Dabarxn}
\title{Varying-payment annuity functions}
\usage{
Iax(x, i, model = NULL, ..., tbl = NULL, k_max = 5000, tol = 1e-12)

Iaxn(x, n, i, model = NULL, ..., tbl = NULL)

Daxn(x, n, i, model = NULL, ..., tbl = NULL)

Iadotx(x, i, model = NULL, ..., tbl = NULL, k_max = 5000, tol = 1e-12)

Iadotxn(x, n, i, model = NULL, ..., tbl = NULL)

Dadotxn(x, n, i, model = NULL, ..., tbl = NULL)

Iabarx(x, i, model, ..., tol = 1e-10)

Iabarxn(x, n, i, model, ...)

Dabarxn(x, n, i, model, ...)
}
\arguments{
\item{x}{Age.}

\item{i}{Effective annual interest rate.}

\item{model}{Optional survival model name.}

\item{...}{Additional model parameters.}

\item{tbl}{Optional life table object for discrete functions.}

\item{k_max}{Maximum summation horizon for non-terminating models.}

\item{tol}{Truncation tolerance for non-terminating models.}

\item{n}{Term in years.}
}
\value{
Numeric vector containing the requested increasing or decreasing annuity value.
}
\description{
Increasing and decreasing life annuity functions.
}
