% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/insurance_discrete.R
\name{insurance_discrete}
\alias{insurance_discrete}
\title{Discrete insurance models}
\description{
Discrete contingent payment and insurance functions.
}
\keyword{internal}
