% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/insurance_utils.R
\name{udd_continuous_multiplier}
\alias{udd_continuous_multiplier}
\title{UDD multiplier for continuous insurance approximations}
\usage{
udd_continuous_multiplier(i)
}
\arguments{
\item{i}{Numeric vector of effective annual interest rates.}
}
\value{
Numeric vector equal to \eqn{i/\delta}.
}
\description{
Under UDD,
\eqn{\bar{A}_x = (i/\delta) A_x}
and similarly for term and deferred insurance.
}
