% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reserve_ch11.R, R/reserve_ch11_ext.R
\name{tsVxn1}
\alias{tsVxn1}
\title{Fractional-duration term reserve}
\usage{
tsVxn1(x, n, t, s, i, tbl = NULL, model = NULL, ...)

tsVxn1(x, n, t, s, i, tbl = NULL, model = NULL, ...)
}
\arguments{
\item{x}{Issue age.}

\item{n}{Term in years.}

\item{t}{Integer duration with t < n.}

\item{s}{Fractional part in [0, 1].}

\item{i}{Effective annual interest rate.}

\item{tbl}{Optional life table object.}

\item{model}{Optional parametric survival model.}

\item{...}{Additional model parameters.}
}
\value{
Numeric vector.

Numeric vector.
}
\description{
Computes the interpolated reserve
\eqn{{}_{t+s}V = ({}_tV + P)(1-s) + {}_{t+1}V \cdot s}
for an n-year term insurance.

Computes the interpolated reserve
\eqn{{}_{t+s}V = ({}_tV + P)(1-s) + {}_{t+1}V \cdot s}
for an n-year term insurance.
}
\examples{
tsVxn1(40, n = 20, t = 10, s = 0.5, i = 0.05, model = "uniform", omega = 100)
tsVxn1(40, n = 20, t = 10, s = 0.5, i = 0.05, model = "uniform", omega = 100)
}
