% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reserve_ch11.R, R/reserve_ch11_ext.R
\name{meanVx}
\alias{meanVx}
\title{Mean reserve for whole life insurance}
\usage{
meanVx(x, t, i, tbl = NULL, model = NULL, ...)

meanVx(x, t, i, tbl = NULL, model = NULL, ...)
}
\arguments{
\item{x}{Issue age.}

\item{t}{Integer contract duration.}

\item{i}{Effective annual interest rate.}

\item{tbl}{Optional life table object.}

\item{model}{Optional parametric survival model.}

\item{...}{Additional model parameters.}
}
\value{
Numeric vector.

Numeric vector.
}
\description{
Computes the mean reserve
\eqn{{}_{t+1/2}V}.

Computes the mean reserve
\eqn{{}_{t+1/2}V}.
}
\examples{
meanVx(40, t = 10, i = 0.05, model = "uniform", omega = 100)
meanVx(40, t = 10, i = 0.05, model = "uniform", omega = 100)
}
