% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/insurance_mthly.R
\name{var_nAx_m}
\alias{var_nAx_m}
\title{Variance of m-thly deferred insurance PV}
\usage{
var_nAx_m(x, n, i, m, model, ..., tol = 1e-12, j_max = 100000L)
}
\arguments{
\item{x}{Age.}

\item{n}{Deferral period.}

\item{i}{Effective annual interest rate.}

\item{m}{Positive integer payment frequency.}

\item{model}{Parametric survival model name.}

\item{...}{Additional model parameters passed to survival-model functions.}

\item{tol}{Numerical tolerance for truncating the infinite sum.}

\item{j_max}{Maximum number of m-thly intervals in the sum.}
}
\value{
Numeric vector of variances.
}
\description{
Variance of m-thly deferred insurance PV
}
