% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chapter18_pensions.R
\name{replacement_ratio_db}
\alias{replacement_ratio_db}
\title{Replacement ratio for a defined benefit plan}
\usage{
replacement_ratio_db(benefit, salary)
}
\arguments{
\item{benefit}{Nonnegative annual retirement benefit.}

\item{salary}{Positive salary measure used in the denominator.}
}
\value{
A numeric vector.
}
\description{
Computes annual benefit divided by a selected salary measure. Scalar
arguments are recycled to a common length.
}
\examples{
replacement_ratio_db(
  benefit = 108008.66,
  salary = 187119.09
)

}
