% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chapter14_multiple_decrement.R
\name{AS_path}
\alias{AS_path}
\title{Projected asset-share path for two decrement causes}
\usage{
AS_path(AS0, G, r, e, b1, b2, q1, q2, p_tau, i, b3 = NULL)
}
\arguments{
\item{AS0}{Initial asset share.}

\item{G}{Premium amount by policy year.}

\item{r}{Percent-of-premium expense rate by policy year.}

\item{e}{Fixed expense by policy year.}

\item{b1}{Cause 1 benefit by policy year.}

\item{b2}{Cause 2 benefit by policy year.}

\item{q1}{Cause 1 probability by policy year.}

\item{q2}{Cause 2 probability by policy year.}

\item{p_tau}{In-force probability by policy year.}

\item{i}{Effective annual interest rate by policy year.}

\item{b3}{Survival benefit by policy year.}
}
\value{
A data frame with policy year \code{k} and asset share \code{AS}.
}
\description{
Convenience interface to \code{AS_path_md()} for two causes.
}
