% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survival_models.R
\name{survival_models}
\alias{survival_models}
\title{Survival models: core survival functions and parametric models}
\description{
This file provides actuarial survival-model utilities.
}
\keyword{internal}
