% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/life_table_select.R
\name{select_life_table}
\alias{select_life_table}
\title{Construct a select life table}
\usage{
select_life_table(x_sel, duration, attained_age, lx)
}
\arguments{
\item{x_sel}{Numeric vector of ages at selection.}

\item{duration}{Numeric vector of durations since selection.}

\item{attained_age}{Numeric vector of attained ages.}

\item{lx}{Numeric vector of select-table survivor values.}
}
\value{
A data frame with class \code{"select_life_table"}.
}
\description{
Builds a select-life-table object from vectors of selection age,
duration since selection, attained age, and survivor values.
}
