% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chapter17_profit_analysis.R
\name{discounted_payback_period}
\alias{discounted_payback_period}
\title{Discounted payback period}
\usage{
discounted_payback_period(Pi, r)
}
\arguments{
\item{Pi}{Numeric profit-signature vector.}

\item{r}{Annual effective risk discount rate. May be scalar or vector;
values must be greater than \code{-1}.}
}
\value{
An integer vector with one value for each discount rate. An element
  is \code{NA_integer_} when payback is not reached.
}
\description{
Returns the first duration at which cumulative discounted profit is
nonnegative.
}
\examples{
Pi <- c(-15.00, 8.42, 8.39, 8.58)
discounted_payback_period(Pi, r = 0.10)

}
