% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reserve_ch10.R
\name{tVx_m}
\alias{tVx_m}
\title{Whole life reserve with m-thly premiums}
\usage{
tVx_m(x, t, m, i, model = NULL, ..., tbl = NULL)
}
\arguments{
\item{x}{Issue age.}

\item{t}{Duration.}

\item{m}{Number of premium payments per year.}

\item{i}{Effective annual interest rate.}

\item{model}{Optional parametric survival model name.}

\item{...}{Additional model parameters.}

\item{tbl}{Optional life table object.}
}
\value{
A numeric vector of values.
}
\description{
Computes the reserve for a whole life insurance funded
by true m-thly premiums.
}
\examples{
tVx_m(40, t = 10, m = 12, i = 0.05, model = "uniform", omega = 100)
}
