% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chapter17_profit_analysis.R
\name{IRR_profit}
\alias{IRR_profit}
\title{Internal rate of return}
\usage{
IRR_profit(Pi, interval = c(0, 1), tol = .Machine$double.eps^0.5)
}
\arguments{
\item{Pi}{Numeric profit-signature vector.}

\item{interval}{Numeric vector of length two giving the root-search
interval. Its lower endpoint must be greater than \code{-1}.}

\item{tol}{Positive scalar tolerance passed to \code{stats::uniroot()}.}
}
\value{
A numeric scalar.
}
\description{
Computes a root of the profit-signature net present value.
}
\examples{
Pi <- c(-15.00, 8.42, 8.39, 8.58)
IRR_profit(Pi)

}
