% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reserve_ch10_ext.R
\name{tVnAdotx}
\alias{tVnAdotx}
\title{Reserve for a deferred annuity-due}
\usage{
tVnAdotx(x, n, t, i, model = NULL, ..., tbl = NULL)
}
\arguments{
\item{x}{Issue age. May be scalar or vector.}

\item{n}{Positive integer deferral period. May be scalar or vector.}

\item{t}{Nonnegative integer duration. May be scalar or vector.}

\item{i}{Effective annual interest rate. May be scalar or vector.}

\item{model}{Optional parametric survival model.}

\item{...}{Additional parameters passed to the actuarial functions.}

\item{tbl}{Optional life table object. Supply by name.}
}
\value{
Numeric vector of reserve values.
}
\description{
Computes the reserve during the deferral period, where \eqn{0 \le t < n}.
}
\examples{
tVnAdotx(
  40,
  n = 20,
  t = 10,
  i = 0.05,
  model = "uniform",
  omega = 100
)
}
