% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decrement_ch13_ext.R
\name{qxtau}
\alias{qxtau}
\title{Total one-year decrement probability}
\usage{
qxtau(qxj)
}
\arguments{
\item{qxj}{Numeric vector of cause-specific decrement probabilities.}
}
\value{
A numeric scalar.
}
\description{
Computes the total one-year multiple-decrement probability
\deqn{q_x^{(\tau)}=\sum_j q_x^{(j)}.}
}
\examples{
qxtau(c(0.011, 0.100))

}
