% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survival_models.R
\name{tqx}
\alias{tqx}
\title{Conditional failure probability}
\usage{
tqx(t, x, model, ...)
}
\arguments{
\item{t}{Numeric vector of durations.}

\item{x}{Numeric vector of ages.}

\item{model}{One of \code{"uniform"}, \code{"exponential"},
\code{"gompertz"}, \code{"makeham"}, or \code{"weibull"}.}

\item{...}{Model parameters.}
}
\value{
Numeric vector of failure probabilities.
}
\description{
Computes \eqn{{}_t q_x = 1 - {}_t p_x}.
}
