% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interest_theory.R
\name{discount}
\alias{discount}
\title{Discount factor for compound interest}
\usage{
discount(i, t)
}
\arguments{
\item{i}{Effective interest rate. May be scalar or vector.}

\item{t}{Time. May be scalar or vector.}
}
\value{
Numeric vector of discount factors.
}
\description{
Computes the discount factor \eqn{v^t = (1+i)^{-t}}.
}
\examples{
discount(0.05, 0:5)
discount(c(0.03, 0.05), 1)
}
