% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chapter18_pensions.R
\name{PAB_fas}
\alias{PAB_fas}
\title{Projected annual benefit under a final-average-salary plan}
\usage{
PAB_fas(x, z, CASx, p, fas_years = 3, past_service = 0, g = NULL, s = NULL)
}
\arguments{
\item{x}{Scalar current or entry age.}

\item{z}{Scalar retirement age.}

\item{CASx}{Positive scalar current annual salary.}

\item{p}{Nonnegative scalar accrual percentage, such as \code{2} for
2 percent.}

\item{fas_years}{Positive integer number of years in the final salary
average.}

\item{past_service}{Nonnegative scalar years of service already completed.}

\item{g}{Optional scalar annual salary growth rate greater than \code{-1}.}

\item{s}{Optional positive salary-scale vector of length \code{z - x}.}
}
\value{
A numeric scalar.
}
\description{
Projects the annual benefit using a final-average-salary formula.
}
\examples{
PAB_fas(
  x = 35,
  z = 65,
  CASx = 60000,
  p = 2,
  fas_years = 3,
  g = 0.04
)

}
