% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reserve_ch11_ext.R
\name{reserve_ch11_ext}
\alias{reserve_ch11_ext}
\title{Extended reserve functions for Chapter 11}
\description{
Additional Chapter 11 reserve functions:
FPT modified reserves, fractional-duration reserves,
gross premium reserves, expense reserves, and
ordered gross gain decomposition.
}
