% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reserve_ch10.R
\name{tVx}
\alias{tVx}
\title{Whole life net level premium reserve}
\usage{
tVx(x, t, i, model = NULL, ..., tbl = NULL)
}
\arguments{
\item{x}{Issue age.}

\item{t}{Duration.}

\item{i}{Effective annual interest rate.}

\item{model}{Optional parametric survival model name.}

\item{...}{Additional model parameters.}

\item{tbl}{Optional life table object.}
}
\value{
A numeric vector of values.
}
\description{
Computes the prospective reserve
for a whole life insurance with annual premiums:
reserve at duration t equals future APV of benefits minus future APV
of net premiums.
}
\examples{
tVx(40, t = 10, i = 0.05, model = "uniform", omega = 100)
}
