% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chapter18_pensions.R
\name{AVz_dc}
\alias{AVz_dc}
\title{Accumulated value of defined contribution plan contributions}
\usage{
AVz_dc(x, z, Sx, c, i, g = NULL, s = NULL)
}
\arguments{
\item{x}{Scalar entry age.}

\item{z}{Scalar retirement age.}

\item{Sx}{Positive scalar salary at age \code{x}.}

\item{c}{Scalar contribution rate in \code{[0, 1]}.}

\item{i}{Scalar annual effective investment return greater than \code{-1}.}

\item{g}{Optional scalar annual salary growth rate greater than \code{-1}.}

\item{s}{Optional positive salary-scale vector of length \code{z - x}.}
}
\value{
A numeric scalar.
}
\description{
Computes the accumulated value at retirement from contributions paid at
the beginning of each year and accumulated to retirement.
}
\examples{
AVz_dc(
  x = 30,
  z = 65,
  Sx = 50000,
  c = 0.10,
  i = 0.05,
  g = 0.04
)

}
