% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reserve_ch10.R
\name{varLtx}
\alias{varLtx}
\title{Variance of present value of loss at duration t for whole life insurance}
\usage{
varLtx(x, t, i, P, model = NULL, ..., tbl = NULL)
}
\arguments{
\item{x}{Issue age.}

\item{t}{Duration.}

\item{i}{Effective annual interest rate.}

\item{P}{Annual premium.}

\item{model}{Optional parametric survival model name.}

\item{...}{Additional model parameters.}

\item{tbl}{Optional life table object.}
}
\value{
A numeric vector of values.
}
\description{
Computes the conditional variance
\eqn{\mathrm{Var}({}_tL_x \mid K_x \ge t)}
for a fully discrete whole life insurance.
}
\examples{
prem <- Px(40, i = 0.05, model = "uniform", omega = 100)
varLtx(40, t = 10, i = 0.05, P = prem, model = "uniform", omega = 100)
}
