% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annuity_annual.R
\name{annuity_annual}
\alias{annuity_annual}
\alias{ax}
\alias{adotx}
\alias{abarx}
\alias{axn}
\alias{adotxn}
\alias{abarxn}
\alias{nax}
\alias{nadotx}
\alias{nabarx}
\alias{sxn}
\alias{sdotxn}
\alias{sbarxn}
\title{Annual annuity functions}
\usage{
ax(x, i, model = NULL, ..., tbl = NULL, k_max = 5000, tol = 1e-12)

adotx(x, i, model = NULL, ..., tbl = NULL, k_max = 5000, tol = 1e-12)

abarx(x, i, model, ..., tol = 1e-10)

axn(x, n, i, model = NULL, ..., tbl = NULL)

adotxn(x, n, i, model = NULL, ..., tbl = NULL)

abarxn(x, n, i, model, ...)

nax(x, n, i, model = NULL, ..., tbl = NULL, k_max = 5000, tol = 1e-12)

nadotx(x, n, i, model = NULL, ..., tbl = NULL, k_max = 5000, tol = 1e-12)

nabarx(x, n, i, model, ..., tol = 1e-10)

sxn(x, n, i, model = NULL, ..., tbl = NULL)

sdotxn(x, n, i, model = NULL, ..., tbl = NULL)

sbarxn(x, n, i, model, ...)
}
\arguments{
\item{x}{Age.}

\item{i}{Effective annual interest rate.}

\item{model}{Optional survival model name.}

\item{...}{Additional model parameters.}

\item{tbl}{Optional life table object for annual discrete annuity functions.}

\item{k_max}{Maximum summation horizon for non-terminating models.}

\item{tol}{Truncation tolerance for non-terminating models.}

\item{n}{Term in years.}
}
\value{
Numeric vector containing the requested annuity present value or actuarial
accumulated value.
}
\description{
Annual whole life, temporary, deferred, and actuarial accumulated value
annuity functions in immediate, due, and continuous forms.

Computes \eqn{a_x = \sum_{t=1}^{\infty} v^t {}_t p_x}.

Computes \eqn{\ddot{a}_x = \sum_{t=0}^{\infty} v^t {}_t p_x}.

Computes \eqn{\bar{a}_x = \int_0^{\infty} v^t {}_t p_x dt}.

Computes \eqn{a_{x:\overline{n}|} = \sum_{t=1}^{n} v^t {}_t p_x}.

Computes \eqn{\ddot{a}_{x:\overline{n}|} =
\sum_{t=0}^{n-1} v^t {}_t p_x}.

Computes \eqn{\bar{a}_{x:\overline{n}|} =
\int_0^n v^t {}_t p_x dt}.

Computes \eqn{{}_{n|}a_x = {}_nE_x a_{x+n}}.

Computes \eqn{{}_{n|}\ddot{a}_x = {}_nE_x \ddot{a}_{x+n}}.

Computes \eqn{{}_{n|}\bar{a}_x = {}_nE_x \bar{a}_{x+n}}.

Computes \eqn{s_{x:\overline{n}|} = a_{x:\overline{n}|} / {}_nE_x}.

Computes \eqn{\ddot{s}_{x:\overline{n}|} =
\ddot{a}_{x:\overline{n}|} / {}_nE_x}.

Computes \eqn{\bar{s}_{x:\overline{n}|} =
\bar{a}_{x:\overline{n}|} / {}_nE_x}.
}
