% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annuity_approximations.R
\name{annuity_approximations_udd}
\alias{annuity_approximations_udd}
\alias{ax_m_udd}
\alias{axn_m_udd}
\alias{nax_m_udd}
\alias{adotx_m_udd}
\alias{adotxn_m_udd}
\alias{nadotx_m_udd}
\alias{sxn_m_udd}
\alias{sdotxn_m_udd}
\alias{abarx_udd}
\alias{abarxn_udd}
\alias{nabarx_udd}
\title{UDD annuity approximations}
\usage{
adotx_m_udd(x, m, i, model, ...)

adotxn_m_udd(x, n, m, i, model, ...)

nadotx_m_udd(x, n, m, i, model, ...)

ax_m_udd(x, m, i, model, ...)

axn_m_udd(x, n, m, i, model, ...)

nax_m_udd(x, n, m, i, model, ...)

sdotxn_m_udd(x, n, m, i, model, ...)

sxn_m_udd(x, n, m, i, model, ...)

abarx_udd(x, i, model, ...)

abarxn_udd(x, n, i, model, ...)

nabarx_udd(x, n, i, model, ...)
}
\arguments{
\item{x}{Age.}

\item{m}{Number of payments per year.}

\item{i}{Effective annual interest rate.}

\item{model}{Survival model.}

\item{...}{Additional model parameters.}

\item{n}{Term.}
}
\value{
Numeric vector.
}
\description{
UDD-based approximations for annual, fractional-payment, and continuous annuity functions.

Computes
\deqn{\ddot{a}_x^{(m)} \approx \alpha(m)\ddot{a}_x - \beta(m).}

Computes
\deqn{\ddot{a}_{x:\overline{n}|}^{(m)} \approx
\alpha(m)\ddot{a}_{x:\overline{n}|} - \beta(m)(1-{}_nE_x).}

Computes
\deqn{{}_{n\mid}\ddot{a}_x^{(m)} \approx
\alpha(m)\,{}_{n\mid}\ddot{a}_x - \beta(m)\,{}_nE_x.}

Computes
\deqn{a_x^{(m)} \approx \alpha(m)a_x + \gamma(m).}

Computes
\deqn{a_{x:\overline{n}|}^{(m)} \approx
\alpha(m)a_{x:\overline{n}|} + \gamma(m)(1-{}_nE_x).}

Computes
\deqn{{}_{n\mid}a_x^{(m)} \approx
\alpha(m)\,{}_{n\mid}a_x + \gamma(m)\,{}_nE_x.}

Computes
\deqn{\ddot{s}_{x:\overline{n}|}^{(m)} \approx
\alpha(m)\ddot{s}_{x:\overline{n}|} - \beta(m)\left(\frac{1}{{}_nE_x}-1\right).}

Computes
\deqn{s_{x:\overline{n}|}^{(m)} \approx
\alpha(m)s_{x:\overline{n}|} + \gamma(m)\left(\frac{1}{{}_nE_x}-1\right).}

Computes
\deqn{\bar{a}_x \approx \frac{id}{\delta^2}\ddot{a}_x - \frac{i-\delta}{\delta^2}.}

Uses the identity
\deqn{\bar{a}_{x:\overline{n}|} \approx \frac{1-\bar{A}_{x:\overline{n}|}}{\delta}}
together with the package's existing Abarxn_udd() approximation.

Computes
\deqn{{}_{n\mid}\bar{a}_x \approx {}_nE_x \, \bar{a}_{x+n}.}
}
\details{
These functions implement the standard Uniform Distribution of Deaths
approximations linking annual, m-thly, and continuous annuity values.

The exported functions documented on this page are:
\itemize{
  \item \code{ax_m_udd()}
  \item \code{axn_m_udd()}
  \item \code{nax_m_udd()}
  \item \code{adotx_m_udd()}
  \item \code{adotxn_m_udd()}
  \item \code{nadotx_m_udd()}
  \item \code{sxn_m_udd()}
  \item \code{sdotxn_m_udd()}
  \item \code{abarx_udd()}
  \item \code{abarxn_udd()}
  \item \code{nabarx_udd()}
}

The abarxn_udd() calculation uses the existing Abarxn_udd() insurance
approximation. Consequently, additional survival-model arguments are not used
in that calculation.
}
