% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/insurance_mthly.R
\name{insurance_mthly}
\alias{insurance_mthly}
\title{m-thly insurance models}
\description{
Exact m-thly contingent payment and insurance functions.
}
\keyword{internal}
