% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chapter18_pensions.R
\name{AAL_PUC_db}
\alias{AAL_PUC_db}
\title{Projected Unit Credit accrued liability for a DB plan}
\usage{
AAL_PUC_db(
  projected_benefit,
  past_service,
  total_service,
  v_to_ret,
  p_surv,
  adue_ret
)
}
\arguments{
\item{projected_benefit}{Projected benefit at retirement.}

\item{past_service}{Past service completed.}

\item{total_service}{Total service at retirement.}

\item{v_to_ret}{Discount factor to retirement.}

\item{p_surv}{Active-service survival probability to retirement.}

\item{adue_ret}{Retirement annuity factor.}
}
\value{
PUC accrued liability.
}
\description{
Computes the PUC accrued liability as the APV of the portion of the
projected benefit attributed to past service.
}
\examples{
AAL_PUC_db(projected_benefit = 30000, past_service = 10, total_service = 30,
v_to_ret = 0.5, p_surv = 0.9, adue_ret = 12)
}
