% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reserve_ch10.R
\name{tVbarAbarx}
\alias{tVbarAbarx}
\title{Fully continuous whole life reserve}
\usage{
tVbarAbarx(x, t, i, model = NULL, ..., tbl = NULL)
}
\arguments{
\item{x}{Issue age.}

\item{t}{Duration, allowed to be any nonnegative numeric value.}

\item{i}{Effective annual interest rate.}

\item{model}{Optional parametric survival model name.}

\item{...}{Additional model parameters.}

\item{tbl}{Optional life table object.}
}
\value{
A numeric vector of values.
}
\description{
Computes the reserve for a whole life insurance with
continuous premiums and immediate payment of claims.
}
\details{
In the fully continuous setting, reserve time \eqn{t} may be any
nonnegative real value.
}
\examples{
tVbarAbarx(40, t = 10, i = 0.05, model = "uniform", omega = 100)
tVbarAbarx(40, t = c(19, 19.25, 19.5, 19.75, 20), i = 0.06,
           model = "uniform", omega = 100)
}
