% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decrement_ch13_ext.R
\name{qx_dep_cf}
\alias{qx_dep_cf}
\title{Multiple-decrement probabilities under constant forces}
\usage{
qx_dep_cf(qxprime)
}
\arguments{
\item{qxprime}{Numeric vector of associated single-decrement
probabilities. Each value must be less than one.}
}
\value{
A numeric vector of dependent cause-specific decrement
  probabilities.
}
\description{
Converts associated single-decrement probabilities to dependent
cause-specific decrement probabilities under constant forces.
}
\examples{
qx_dep_cf(c(0.20, 0.10))

}
