% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mortality_improvement.R
\name{naxn_improved}
\alias{naxn_improved}
\title{Deferred temporary annuity-immediate under mortality improvement}
\usage{
naxn_improved(x0, u, n, i, qx_base_vec, AAx_vec, base_year, issue_year)
}
\arguments{
\item{x0}{Issue age.}

\item{u}{Deferral period in years.}

\item{n}{Temporary payment period in years.}

\item{i}{Effective annual interest rate.}

\item{qx_base_vec}{Base-year one-year death probabilities for ages
\code{x0, x0+1, ..., x0+u+n-1}.}

\item{AAx_vec}{Mortality improvement factors for ages
\code{x0, x0+1, ..., x0+u+n-1}.}

\item{base_year}{Base year.}

\item{issue_year}{Issue year.}
}
\value{
Numeric scalar.
}
\description{
Computes
\deqn{{}_{u\mid}a_{x:\overline{n}|} = \sum_{t=u+1}^{u+n} v^t \, {}_tp_x^{(\mathrm{improved})}}
}
