% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survival_models.R
\name{ex_complete}
\alias{ex_complete}
\title{Complete expectation of life}
\usage{
ex_complete(x, model, ..., tol = 1e-10)
}
\arguments{
\item{x}{Numeric vector of ages.}

\item{model}{One of \code{"uniform"}, \code{"exponential"},
\code{"gompertz"}, \code{"makeham"}, or \code{"weibull"}.}

\item{...}{Model parameters.}

\item{tol}{Tolerance used to choose a finite integration bound.}
}
\value{
Numeric vector of complete expectations.
}
\description{
Computes \eqn{\overset{\circ}{e}_x=\int_0^\infty {}_t p_x\,dt}.
}
