% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/numerical_utils.R
\name{.find_upper_t}
\alias{.find_upper_t}
\title{Internal: choose a practical upper bound for integrals to infinity}
\usage{
.find_upper_t(x, model, ..., eps = 1e-10, t_max = 500)
}
\description{
Strategy: find t such that S0(t) is very small, or fall back to a cap.
}
\keyword{internal}
