% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/survival_models.R
\name{tpx}
\alias{tpx}
\title{Conditional survival probability}
\usage{
tpx(t, x, model, ...)
}
\arguments{
\item{t}{Numeric vector of durations.}

\item{x}{Numeric vector of ages.}

\item{model}{One of \code{"uniform"}, \code{"exponential"},
\code{"gompertz"}, \code{"makeham"}, or \code{"weibull"}.}

\item{...}{Model parameters.}
}
\value{
Numeric vector of survival probabilities.
}
\description{
Computes \eqn{{}_t p_x = S_0(x+t)/S_0(x)}.
}
