% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reserve_ch10.R
\name{tVxn1}
\alias{tVxn1}
\title{Term insurance net level premium reserve}
\usage{
tVxn1(x, n, t, i, model = NULL, ..., tbl = NULL)
}
\arguments{
\item{x}{Issue age.}

\item{n}{Term in years.}

\item{t}{Duration.}

\item{i}{Effective annual interest rate.}

\item{model}{Optional parametric survival model name.}

\item{...}{Additional model parameters.}

\item{tbl}{Optional life table object.}
}
\value{
A numeric vector of values.
}
\description{
Computes the prospective reserve for an n-year term insurance.
}
\examples{
tVxn1(40, n = 20, t = 10, i = 0.05, model = "uniform", omega = 100)
}
