% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mortality_improvement.R
\name{ax_improved}
\alias{ax_improved}
\title{Whole life annuity-immediate under mortality improvement}
\usage{
ax_improved(x0, i, qx_base_vec, AAx_vec, base_year, issue_year)
}
\arguments{
\item{x0}{Issue age.}

\item{i}{Effective annual interest rate.}

\item{qx_base_vec}{Base-year one-year death probabilities for successive ages.}

\item{AAx_vec}{Mortality improvement factors for successive ages.}

\item{base_year}{Base year.}

\item{issue_year}{Issue year.}
}
\value{
Numeric scalar.
}
\description{
Computes a truncated whole life annuity-immediate under projected mortality.
}
