% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/insurance_discrete.R
\name{nEx}
\alias{nEx}
\title{Pure endowment APV}
\usage{
nEx(x, n, i, tbl = NULL, model = NULL, ...)
}
\arguments{
\item{x}{Age.}

\item{n}{Term.}

\item{i}{Effective annual interest rate.}

\item{tbl}{Optional life table object.}

\item{model}{Optional parametric survival model name.}

\item{...}{Additional arguments passed to survival-model functions.}
}
\value{
Numeric vector of APVs.
}
\description{
Computes \eqn{{}_nE_x = v^n {}_n p_x}.
}
