% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multilife_ch12.R
\name{axybar_ch12}
\alias{axybar_ch12}
\alias{axybar}
\title{Last-survivor whole life annuity-immediate}
\usage{
axybar(x, y, i, tbl = NULL, model = NULL, ..., k_max = 5000, tol = 1e-12)
}
\arguments{
\item{x}{Age of first life.}

\item{y}{Age of second life.}

\item{i}{Effective annual interest rate.}

\item{tbl}{Life table.}

\item{model}{Survival model.}

\item{...}{Additional model parameters.}

\item{k_max}{Maximum number of terms.}

\item{tol}{Convergence tolerance.}
}
\value{
Numeric vector.
}
\description{
Computes \eqn{a_{\overline{xy}}}.
}
\details{
Shared documentation topic used to avoid filename collisions with
case-distinct function names on case-insensitive file systems.
}
\examples{
axybar(40, 50, i = 0.05, model = "uniform", omega = 100)
}
