% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/insurance_varying_benefits.R
\name{DAxn1}
\alias{DAxn1}
\title{Decreasing n-year term insurance}
\usage{
DAxn1(x, n, i, tbl = NULL, model = NULL, ...)
}
\arguments{
\item{x}{Age.}

\item{n}{Term.}

\item{i}{Effective annual interest rate.}

\item{tbl}{Optional life table object.}

\item{model}{Optional parametric survival model.}

\item{...}{Additional model parameters.}
}
\value{
Numeric vector.
}
\description{
Computes
\deqn{(DA)_{x:\overline{n}|}^{1}
= \sum_{k=0}^{n-1} (n-k) v^{k+1} \Pr(K_x = k).}
}
