% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chapter16_universal_life.R
\name{GMF_rollforward_ul}
\alias{GMF_rollforward_ul}
\title{Guaranteed maturity fund roll-forward}
\usage{
GMF_rollforward_ul(GMF_prev, GMP, r, policy_charge, i)
}
\arguments{
\item{GMF_prev}{Prior guaranteed maturity fund.}

\item{GMP}{Guaranteed maturity premium.}

\item{r}{Percent-of-premium expense rate in \code{[0, 1]}.}

\item{policy_charge}{Guaranteed policy charge.}

\item{i}{Guaranteed annual effective interest rate greater than \code{-1}.}
}
\value{
A numeric vector.
}
\description{
Computes a one-period guaranteed maturity fund roll-forward. Arguments may
be scalars or vectors and follow common-length recycling.
}
\examples{
GMF_rollforward_ul(140.40, 14.49, 0.04, 11.80, 0.03)

}
