% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chapter18_pensions.R
\name{replacement_ratio_dc}
\alias{replacement_ratio_dc}
\title{Replacement ratio for a defined contribution plan}
\usage{
replacement_ratio_dc(x, z, Sx, c, i, adue_z, g = NULL, s = NULL)
}
\arguments{
\item{x}{Scalar entry age.}

\item{z}{Scalar retirement age.}

\item{Sx}{Positive scalar salary at age \code{x}.}

\item{c}{Scalar contribution rate in \code{[0, 1]}.}

\item{i}{Scalar annual effective investment return greater than \code{-1}.}

\item{adue_z}{Positive scalar whole-life annuity-due factor at retirement.}

\item{g}{Optional scalar annual salary growth rate greater than \code{-1}.}

\item{s}{Optional positive salary-scale vector of length \code{z - x}.}
}
\value{
A numeric scalar.
}
\description{
Computes annual retirement income divided by salary in the final
pre-retirement year.
}
\examples{
replacement_ratio_dc(
  x = 30,
  z = 65,
  Sx = 50000,
  c = 0.10,
  i = 0.05,
  adue_z = 12,
  g = 0.04
)

}
