% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chapter18_pensions.R
\name{AAL_TUC_db}
\alias{AAL_TUC_db}
\title{Traditional Unit Credit accrued liability}
\usage{
AAL_TUC_db(accrued_benefit, v_to_ret, p_surv, adue_ret)
}
\arguments{
\item{accrued_benefit}{Nonnegative accrued benefit.}

\item{v_to_ret}{Nonnegative discount factor from the valuation date to
retirement.}

\item{p_surv}{Survival or active-service probability to retirement in
\code{[0, 1]}.}

\item{adue_ret}{Positive retirement annuity-due factor.}
}
\value{
A numeric vector.
}
\description{
Computes the actuarial present value of the benefit accrued through the
valuation date.
}
\examples{
AAL_TUC_db(
  accrued_benefit = 12000,
  v_to_ret = 0.5,
  p_surv = 0.9,
  adue_ret = 12
)

}
