% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chapter14_multiple_decrement.R
\name{thiele_dVdt_01}
\alias{thiele_dVdt_01}
\title{Reserve derivatives for a disability model with recovery}
\usage{
thiele_dVdt_01(t, V0, V1, delta, Pbar, B, R, mu01, mu02, mu10, mu12)
}
\arguments{
\item{t}{Time.}

\item{V0}{Healthy-state reserve.}

\item{V1}{Disabled-state reserve.}

\item{delta}{Force of interest.}

\item{Pbar}{Continuous premium rate.}

\item{B}{Death benefit.}

\item{R}{Continuous disability income rate.}

\item{mu01}{Healthy-to-disabled intensity function.}

\item{mu02}{Healthy-to-deceased intensity function.}

\item{mu10}{Disabled-to-healthy intensity function.}

\item{mu12}{Disabled-to-deceased intensity function.}
}
\value{
A named vector for scalar input or a two-column matrix for
  vectorized input.
}
\description{
Computes the coupled Thiele reserve derivatives. Numeric arguments may be
scalar or compatible vectors.
}
