% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decrement_ch13_ext.R
\name{tqxprime_mudd}
\alias{tqxprime_mudd}
\title{Fractional-year associated single-decrement probabilities under MUDD}
\usage{
tqxprime_mudd(qxj, t)
}
\arguments{
\item{qxj}{Numeric vector of dependent cause-specific decrement
probabilities.}

\item{t}{A single time in \eqn{[0,1]}.}
}
\value{
A numeric vector of associated single-decrement probabilities.
}
\description{
Computes the associated single-decrement probabilities through time
\code{t} under the multiple-decrement uniform distribution assumption.
}
\examples{
tqxprime_mudd(c(0.20, 0.10), t = 0.5)

}
