% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reserve_ch11.R, R/reserve_ch11_ext.R
\name{tVGx}
\alias{tVGx}
\title{Whole life gross premium reserve}
\usage{
tVGx(
  x,
  t,
  i,
  G,
  benefit = 1,
  renewal_premium_pct = 0,
  renewal_policy_exp = 0,
  settlement_exp = 0,
  tbl = NULL,
  model = NULL,
  ...
)

tVGx(
  x,
  t,
  i,
  G,
  benefit = 1,
  renewal_premium_pct = 0,
  renewal_policy_exp = 0,
  settlement_exp = 0,
  tbl = NULL,
  model = NULL,
  ...
)
}
\arguments{
\item{x}{Issue age.}

\item{t}{Duration.}

\item{i}{Effective annual interest rate.}

\item{G}{Gross annual premium.}

\item{benefit}{Insurance amount. Default 1.}

\item{renewal_premium_pct}{Renewal percent-of-premium expense.}

\item{renewal_policy_exp}{Renewal per-policy expense.}

\item{settlement_exp}{Settlement expense at death.}

\item{tbl}{Optional life table object.}

\item{model}{Optional parametric survival model.}

\item{...}{Additional model parameters.}
}
\value{
Numeric vector.

Numeric vector.
}
\description{
Computes the Chapter 11 prospective gross premium reserve for a
fully discrete whole life insurance with annual premiums and
renewal expenses.

Computes the Chapter 11 prospective gross premium reserve for a
fully discrete whole life insurance with annual premiums and
renewal expenses.
}
\details{
This function is intended for durations after issue, where future
expenses are modeled through renewal premium expenses, renewal
per-policy expenses, and settlement expense.
}
\examples{
tVGx(
  x = 40, t = 10, i = 0.05, G = 0.03,
  benefit = 1, renewal_premium_pct = 0.10,
  renewal_policy_exp = 0.002, settlement_exp = 0.02,
  model = "uniform", omega = 100
)
tVGx(
  x = 40, t = 10, i = 0.05, G = 0.03,
  benefit = 1, renewal_premium_pct = 0.10,
  renewal_policy_exp = 0.002, settlement_exp = 0.02,
  model = "uniform", omega = 100
)
}
