% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mortality_improvement.R
\name{mortality_improvement_projection}
\alias{mortality_improvement_projection}
\alias{qx_proj}
\alias{px_proj}
\alias{tpx_improved}
\alias{axn_improved}
\alias{naxn_improved}
\alias{ax_improved}
\title{Mortality improvement projection functions}
\usage{
qx_proj(qx_base, AAx, base_year, proj_year)

px_proj(qx_base, AAx, base_year, proj_year)

tpx_improved(x0, n, qx_base_vec, AAx_vec, base_year, issue_year)

axn_improved(x0, n, i, qx_base_vec, AAx_vec, base_year, issue_year)

naxn_improved(x0, u, n, i, qx_base_vec, AAx_vec, base_year, issue_year)

ax_improved(x0, i, qx_base_vec, AAx_vec, base_year, issue_year)
}
\arguments{
\item{qx_base}{Base-year one-year death probability.}

\item{AAx}{Mortality improvement factor.}

\item{base_year}{Base year.}

\item{proj_year}{Projection year. May be scalar or vector.}

\item{x0}{Issue age.}

\item{n}{Number of years.}

\item{qx_base_vec}{Base-year death probabilities for successive ages.}

\item{AAx_vec}{Mortality improvement factors for successive ages.}

\item{issue_year}{Issue year.}

\item{i}{Effective annual interest rate.}

\item{u}{Deferral period in years.}
}
\value{
Numeric vector of projected one-year death probabilities.
}
\description{
Functions for projecting one-year death and survival probabilities under
mortality improvement and evaluating annuity values under projected
mortality.
}
\details{
The standard projection used is
\deqn{q_x^{[Y]} = q_x^{[B]} (1 - AA_x)^{Y-B},}
where \eqn{B} is the base year, \eqn{Y} is the projection year, and
\eqn{AA_x} is the mortality improvement factor at age \eqn{x}.
}
