% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/multilife_ch12_ext.R
\name{abarx_y}
\alias{abarx_y}
\title{Continuous reversionary annuity to (y) after death of (x)}
\usage{
abarx_y(x, y, i, tbl = NULL, model = NULL, ...)
}
\arguments{
\item{x}{Age of first life.}

\item{y}{Age of second life.}

\item{i}{Effective annual interest rate.}

\item{tbl}{Life table.}

\item{model}{Survival model.}

\item{...}{Additional model parameters.}
}
\value{
Numeric vector.
}
\description{
Computes \eqn{\overline{a}_{x|y} = \overline{a}_y - \overline{a}_{xy}}.
}
\examples{
abarx_y(40, 50, i = 0.05, model = "uniform", omega = 100)
}
