% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/decrement_ch13_ext.R
\name{tpx_tau_cf}
\alias{tpx_tau_cf}
\title{Total survival under constant cause-specific forces}
\usage{
tpx_tau_cf(mu, t)
}
\arguments{
\item{mu}{Numeric vector of nonnegative cause-specific forces.}

\item{t}{Nonnegative time. May be scalar or vector.}
}
\value{
A numeric vector.
}
\description{
Computes
\deqn{
{}_tp_x^{(\tau)}
=
\exp\left(-t\sum_j\mu_j\right).
}
}
\examples{
tpx_tau_cf(c(0.10, 0.20), 5)
tpx_tau_cf(c(0.10, 0.20), c(1, 5, 10))

}
