% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reserve_ch11.R
\name{fractional_duration_reserves}
\alias{fractional_duration_reserves}
\alias{tsVx}
\alias{meanVx}
\title{Fractional-duration whole life reserves}
\usage{
tsVx(x, t, s, i, tbl = NULL, model = NULL, ...)

meanVx(x, t, i, tbl = NULL, model = NULL, ...)
}
\arguments{
\item{x}{Issue age. May be scalar or vector.}

\item{t}{Nonnegative integer duration. May be scalar or vector.}

\item{s}{Fractional duration in \eqn{[0,1]}. May be scalar or vector.}

\item{i}{Effective annual interest rate. May be scalar or vector.}

\item{tbl}{Optional life table object.}

\item{model}{Optional parametric survival model.}

\item{...}{Additional parameters passed to the actuarial functions.}
}
\value{
A numeric vector of reserve values.
}
\description{
Computes fractional-duration whole life reserves using linear
interpolation between the reserve immediately after the premium at
duration \eqn{t} and the reserve at duration \eqn{t+1}.
}
\details{
\code{tsVx()} computes the reserve at fractional duration
\eqn{t+s}, where \eqn{0 \le s \le 1}.

\code{meanVx()} computes the reserve at the midpoint of the policy
year (\eqn{s=0.5}).


The fractional reserve is computed using

\deqn{
{}_{t+s}V_x
=
({}_tV_x + P_x)(1-s)
+
{}_{t+1}V_x s.
}

The function \code{meanVx()} is a convenience wrapper corresponding to
\eqn{s=0.5}.
}
\examples{
tsVx(
  40,
  t = 10,
  s = 0.5,
  i = 0.05,
  model = "uniform",
  omega = 100
)

meanVx(
  40,
  t = 10,
  i = 0.05,
  model = "uniform",
  omega = 100
)

}
