% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/insurance_continuous.R
\name{var_Abarx}
\alias{var_Abarx}
\title{Variance of continuous whole life insurance PV}
\usage{
var_Abarx(x, i, model, ...)
}
\arguments{
\item{x}{Age.}

\item{i}{Effective annual interest rate.}

\item{model}{Parametric survival model name.}

\item{...}{Additional model parameters passed to survival-model functions.}
}
\value{
Numeric vector of variances.
}
\description{
Computes \eqn{\mathrm{Var}(\bar{Z}_x) =
{}^{2}\bar{A}_x - \bar{A}_x^2}.
}
