% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/insurance_varying_benefits.R
\name{IAx}
\alias{IAx}
\title{Increasing whole life insurance}
\usage{
IAx(x, i, tbl = NULL, model = NULL, ..., tol = 1e-12, k_max = 5000)
}
\arguments{
\item{x}{Age.}

\item{i}{Effective annual interest rate.}

\item{tbl}{Optional life table object.}

\item{model}{Optional parametric survival model.}

\item{...}{Additional model parameters.}

\item{tol}{Numerical tolerance for truncation.}

\item{k_max}{Maximum number of terms.}
}
\value{
Numeric vector.
}
\description{
Computes
\deqn{(IA)_x = \sum_{k=0}^{\infty} (k+1) v^{k+1} \Pr(K_x = k).}
}
