% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reserve_ch11.R, R/reserve_ch11_ext.R
\name{betaF}
\alias{betaF}
\title{Full preliminary term renewal modified premium}
\usage{
betaF(x, i, tbl = NULL, model = NULL, ...)

betaF(x, i, tbl = NULL, model = NULL, ...)
}
\arguments{
\item{x}{Issue age.}

\item{i}{Effective annual interest rate.}

\item{tbl}{Optional life table object.}

\item{model}{Optional parametric survival model.}

\item{...}{Additional model parameters.}
}
\value{
Numeric vector.

Numeric vector.
}
\description{
Computes the FPT renewal premium
\eqn{\beta^F = P_{x+1}}
for whole life insurance.

Computes the FPT renewal premium
\eqn{\beta^F = P_{x+1}}
for whole life insurance.
}
\examples{
betaF(40, i = 0.05, model = "uniform", omega = 100)
betaF(40, i = 0.05, model = "uniform", omega = 100)
}
