% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reserve_ch11.R
\name{reserve_ch11}
\alias{reserve_ch11}
\title{Reserve functions for Chapter 11}
\description{
Chapter 11 functions for modified reserves, fractional-duration reserves,
gross premium reserves, expense reserves, and gross gain analysis.
}
