% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/insurance_continuous.R
\name{A2barx}
\alias{A2barx}
\title{Second moment of continuous whole life insurance PV}
\usage{
A2barx(x, i, model, ...)
}
\arguments{
\item{x}{Age.}

\item{i}{Effective annual interest rate.}

\item{model}{Parametric survival model name.}

\item{...}{Additional model parameters passed to survival-model functions.}
}
\value{
Numeric vector of second moments.
}
\description{
Computes \eqn{{}^{2}\bar{A}_x} by evaluating \eqn{\bar{A}_x}
at doubled force.
}
