% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chapter17_profit_analysis.R
\name{Pi_signature}
\alias{Pi_signature}
\title{Profit signature}
\usage{
Pi_signature(Pr, p_tau)
}
\arguments{
\item{Pr}{Profit vector of length \code{n + 1}.}

\item{p_tau}{One-year in-force probabilities. For \code{n > 1}, this may
have length \code{n - 1} or \code{n}; the final value is ignored when
length \code{n}. For a one-year contract, use \code{numeric(0)} or a
single probability.}
}
\value{
A named numeric vector with the same length as \code{Pr}.
}
\description{
Converts policy-year expected profits into a profit signature by weighting
each future expected profit by the probability that the contract is in
force at the start of that policy year.
}
\examples{
Pr <- c(-15.00, 8.42, 8.40, 8.61)
Pi_signature(Pr, p_tau = c(0.99858, 0.99847, 0.99834))

}
