% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reserve_ch10_ext.R
\name{Pnax}
\alias{Pnax}
\title{Net premium for a deferred annuity-immediate}
\usage{
Pnax(x, n, i, model = NULL, ..., tbl = NULL)
}
\arguments{
\item{x}{Issue age. May be scalar or vector.}

\item{n}{Positive integer deferral period. May be scalar or vector.}

\item{i}{Effective annual interest rate. May be scalar or vector.}

\item{model}{Optional parametric survival model.}

\item{...}{Additional parameters passed to the actuarial functions.}

\item{tbl}{Optional life table object. Supply by name.}
}
\value{
Numeric vector of net annual premiums.
}
\description{
Computes
\deqn{
P({}_{n|}a_x)
=
\frac{{}_{n|}a_x}
{\ddot{a}_{x:\overline{n}|}}.
}
}
\examples{
Pnax(
  40,
  n = 20,
  i = 0.05,
  model = "uniform",
  omega = 100
)
}
