% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chapter16_universal_life.R
\name{iP_eiul}
\alias{iP_eiul}
\title{Point-to-point index growth rates}
\usage{
iP_eiul(index)
}
\arguments{
\item{index}{Numeric vector of strictly positive index values.}
}
\value{
A numeric vector with length one less than \code{index}.
}
\description{
Computes consecutive point-to-point growth rates from index values.
}
\examples{
iP_eiul(c(1000, 1050, 1200, 1100))

}
