% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reserve_ch11.R
\name{GTg_disc}
\alias{GTg_disc}
\title{Total gross gain for a discrete insurance contract}
\usage{
GTg_disc(
  VtG,
  Vt1G,
  G,
  i_actual,
  q_actual,
  r_actual = 0,
  e_actual = 0,
  s_actual = 0,
  b = 1
)
}
\arguments{
\item{VtG}{Gross reserve at duration \code{t}.}

\item{Vt1G}{Gross reserve at duration \code{t + 1}.}

\item{G}{Gross premium.}

\item{i_actual}{Actual annual effective interest rate.}

\item{q_actual}{Actual mortality probability.}

\item{r_actual}{Actual percent-of-premium expense rate.}

\item{e_actual}{Actual per-policy expense.}

\item{s_actual}{Actual settlement expense.}

\item{b}{Benefit amount.}
}
\value{
A numeric vector of total gains.
}
\description{
Computes total gain during one policy year under gross premiums,
gross reserves, actual mortality, actual interest, and actual expenses.
}
\examples{
GTg_disc(
  VtG = 0.10,
  Vt1G = 0.12,
  G = 0.02,
  i_actual = 0.05,
  q_actual = 0.01,
  r_actual = 0.03,
  e_actual = 0,
  s_actual = 0.01,
  b = 1
)

}
