% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chapter18_pensions.R
\name{PAB_cae}
\alias{PAB_cae}
\title{Projected annual benefit under a career-average-earnings plan}
\usage{
PAB_cae(x, z, CASx, p, past_salary_total = 0, g = NULL, s = NULL)
}
\arguments{
\item{x}{Scalar current or entry age.}

\item{z}{Scalar retirement age.}

\item{CASx}{Positive scalar current annual salary.}

\item{p}{Nonnegative scalar accrual percentage.}

\item{past_salary_total}{Nonnegative total of actual prior salaries.}

\item{g}{Optional scalar annual salary growth rate greater than \code{-1}.}

\item{s}{Optional positive salary-scale vector of length \code{z - x}.}
}
\value{
A numeric scalar.
}
\description{
Projects the annual benefit using a career-average-earnings formula.
}
\examples{
PAB_cae(
  x = 30,
  z = 65,
  CASx = 100000,
  p = 1,
  g = 0.04
)

}
