% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/insurance_continuous.R
\name{A2nAbarx}
\alias{A2nAbarx}
\title{Second moment of continuous deferred insurance PV}
\usage{
A2nAbarx(x, n, i, model, ...)
}
\arguments{
\item{x}{Age.}

\item{n}{Deferral period.}

\item{i}{Effective annual interest rate.}

\item{model}{Parametric survival model name.}

\item{...}{Additional model parameters passed to survival-model functions.}
}
\value{
Numeric vector of second moments.
}
\description{
Computes \eqn{{}^{2}{}_{n\mid}\bar{A}_x} by evaluating
\eqn{{}_{n\mid}\bar{A}_x} at doubled force.
}
