% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/numerical_utils.R
\name{.safe_integrate}
\alias{.safe_integrate}
\title{Internal: safe numerical integration helper}
\usage{
.safe_integrate(f, lower, upper, ..., rel.tol = 1e-10)
}
\description{
Internal: safe numerical integration helper
}
\keyword{internal}
