% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mortality_improvement.R
\name{axn_improved}
\alias{axn_improved}
\title{Temporary annuity-immediate under mortality improvement}
\usage{
axn_improved(x0, n, i, qx_base_vec, AAx_vec, base_year, issue_year)
}
\arguments{
\item{x0}{Issue age.}

\item{n}{Term in years.}

\item{i}{Effective annual interest rate.}

\item{qx_base_vec}{Base-year one-year death probabilities for ages
\code{x0, x0+1, ..., x0+n-1}.}

\item{AAx_vec}{Mortality improvement factors for ages
\code{x0, x0+1, ..., x0+n-1}.}

\item{base_year}{Base year.}

\item{issue_year}{Issue year.}
}
\value{
Numeric scalar.
}
\description{
Computes
\deqn{a_{x:\overline{n}|} = \sum_{t=1}^n v^t \, {}_tp_x^{(\mathrm{improved})}}
using projected mortality rates.
}
